import AL_Khatma
from .log import log
import json
import requests
import os
from tqdm import tqdm
import urllib3
from time import perf_counter

__main_path__ = os.path.dirname(__file__)

class Quran:
    def __init__(self, lang='main'):
        """
        Quran(class): 
        lang: اللغة التي تريد عرضها
        """
        try:
            #* قراءة ملف اللغات
            os.chdir(path=__main_path__)
            self.quran = json.load(open(f"./DATA/Language/{lang}.json", "r", encoding="utf8"))
            log(
                f'{__file__} > Quran | Status JSON File ', 
                f'Read: True, Language: {lang}'
                ).write_message()
        except FileNotFoundError as e:
            #* في حال طلب المستخدم استعراض اللغات

            if lang == '?':
                log(
                f'{__file__} > Quran | Show All Language ', 
                f'The user has requested to view the available languages of the Quran'
                ).write_message()

                lang = os.listdir(path='./DATA/Language')
                print('the available languages of the Quran'.title())
                for i in range(len(lang)):
                    nameFile = lang[i].split(".")[0]
                    if nameFile == 'main': print(f"[{i}] ar, en ({nameFile})")
                    else: print(f"[{i}] {nameFile}")
            else:
                log(
                f'{__file__} > Quran | Status JSON File ', 
                f'Falus | The user has been entered for an unavailable language'
                ).write_message()
                
                print('[Quran] Sorry, the language is your enter is not available.\nthe available languages of the Quran:')
                
                #// إنتبه: قم بتغير مسار لكي لا تحصل مشاكل
                #// قم بوضع طريقة لقراءة جميع الملفات دون مشاكل عدم معرفه موقعها
                lang = os.listdir(path='./DATA/Language')               
                for i in range(len(lang)):
                    nameFile = lang[i].split(".")[0]
                    if nameFile == 'main': print(f"[{i}] ar, en ({nameFile})")
                    else: print(f"[{i}] {nameFile}")


    def show_block_aya(
            self,
            verses_no:int,
            verses_number:list,
            orderly=False,
            ):
        """
        show_block_aya(func): هي دالة تقوم بإستخراج ما يدرده المستخدم من ملف اللغة
        verses_no(int): أستخراج السروة التي تريدها
        verses_number(list): في حالة أنك ترد صورة بعينها يمكنك ذلك بكتابة رقم أيتها. ويمكن أن تجعلعا فارغة
        orderly(bool): إذا أردت أن ترى النتيجة مطبوعة بشكل يمكن قراءتها
        """
        log(
            "quran.py > Quran > show_block_aya | info",
            f"verses_no: {verses_no}, verses_number: {verses_number}, orderly: {orderly}"
        ).write_message()

        if verses_no <= 114:
            log(
            f"{__file__} > Quran > show_block_aya | number of Surahs of the Qur’an is good",
            f"The number entered did not exceed the number of Surahs of the Qur’an"
            ).write_message()
            #? لتدوين أيات جميع السورة المحددة
            results = list()
            #? تدوين أعداد السورة المطلوبة
            verses = list()
            #* يحدث عملية البحث عن السورة المطلوبة
            for search in range(len(self.quran)):
                #* التأكد من السورة المطلوبة من ترتيبها
                if self.quran[search]['verses_no'] == verses_no:
                    
                    #* تسجيل جميع الأيات السورة في قائمة + تسجيل الأيات المطلوبة في قائمة منفصلة(إن وجد)
                    for i in range(self.quran[search]['total_verses']):
                        results.append(self.quran[search+i])

                        #? هنا تحدث عملية رصد الأيات المطلوبة تزامُنا مع علميلة رصد أيات السورة المطلوبة
                        for cheaking in verses_number:
                            #? التحقق من أن الأية المطلوبة ضمن عدد أيات السورة + التأكد من الأية بعينها لإضافتها في قائمة مخصصة
                            if self.quran[search+i]['total_verses'] >= cheaking and cheaking == self.quran[search+i]['verses_number']:
                                verses.append(self.quran[search+i])
                            else:
                                continue

                    #* طباعة تقرير مُنسق مع إرجاع قينتين للعمليتين السابقتين
                    if orderly:
                        txt = ('results'.upper(), 'verses'.upper())
                        print(f'\n\n\n{txt[0]:.^50}\n\n\n')
                        print(json.dumps(results, indent=4, ensure_ascii=False))
                        print(f'\n\n\n{txt[1]:.^50}\n\n\n')
                        print(json.dumps(verses, indent=4, ensure_ascii=False))
                        
                        return (results, verses)
                    #* إرجاع القيمتين السابقين فقط
                    else: return (results, verses)
                #? هذا في حالة عدم إجاد السورة فإنه يستمر حتى يجد السورة المطلوب
                else: continue
        #? في حالة تجاوز 114 سورة قرأنية يتم رفض الطلب
        else:
            log(
            f"{__file__} > Quran > show_block_aya | Verses Over Error",
            f"I have exceeded 114 Quranic chapters. Please adhere to the number of Quranic chapters (from 1 to 114)."
            ).write_message()
            print('I have exceeded 114 Quranic chapters. Please adhere to the number of Quranic chapters (from 1 to 114).')
            return (False,)


    #* هذا للبحث عن كلمة أو مجموعة كلمات في القرآن
    def searching(
            self, 
            text:str, 
            search_second_lang=False, 
            print_report=False
            ):
        """
        searching(func): هي دالة للبحث بين السطور القرآنية
        text(str): الكلمة المراد بحثها
        search_second_lang(bool): في حال تفعيل هذه الخاصية سيتم البحث بنائا على اللغة الثانية في ملف اللغة
        print_report(bool): في حال تفعيل هذه الخاصية قوم بطباعة تقرير منظم عن نتائج البحث
        """
        log(
            f"{__file__} > Quran > searching | Search Info",
            f"Search: {text}, search_my_lang: {search_second_lang}, print_report: {print_report}"
            ).write_message()
        start = perf_counter()
        #? في حال إذا كان النص فارغ لا يتم تنفيذ الطلب
        if text != '':
            log(
            f"{__file__} > Quran > searching | The Search Status",
            f"True"
            ).write_message()
            end = perf_counter()
            # * يكون جميع النص lower
            text    = text.lower()
            #* لتحزين النتائج
            results = list()
            #? في حال إذا كان خيار search_my_lang مفعل، تقوم خوارزمية للكشف عن نص اللغة الأخرى
            lang    = ['English', 'Bengali', 'Chinese', 'Spanish', 'French', 'Indonesian', 'Russian', 'Swedish', 'Turkish', 'Urdu']
            if search_second_lang:
                try:
                    #* استكشاف اللغة المناسبة من ملف اللغة
                    for LANG in lang:
                        lang_ues    = f'verses_text_{LANG.lower()}'
                        verses_text = self.quran[0][lang_ues]
                        break
                except KeyError:
                    lang_ues = 'verses_text_without_diacritical'

            #* تبدأ هنا عملية البحث
            for i in range(len(self.quran)):
                #? يتم تطبيق هذا الأمر في حالة تفعيل خيار search_my_langsearch_my_lang
                if search_second_lang:
                    verses_text = self.quran[i][lang_ues].lower()
                    search      = verses_text.find(text)
                #? في حال عدم تفعيل search_my_lang فإنهُ يتم تنفيذ الأمر
                else:
                    verses_text = self.quran[i]['verses_text_without_diacritical']
                    search      = verses_text.find(text)
                #* التحقق من عملية البحث
                #? -1 يمثل أن قيمة البحث سلبية، وهذا يعني أن النص المطلوب غير متوفر
                if search != -1:
                    results.append(self.quran[i])
                else:
                    continue
            #* إرجاع قيمة البحث
            end = perf_counter()
            if print_report: print(f"[REPORT]\nRuning Time: {end-start}\nCount Search: {len(results)}\nResult Search: {json.dumps(results, indent=4, ensure_ascii=False)}")
            return results
        
        else:
            log(
            f"{__file__} > Quran > searching | The Search Status",
            f"The search failed; the user has not made any input in search zone or input empty."
            ).write_message()
            end = perf_counter()
            if print_report: print(f"[REPORT]\nRuning Time: {end-start}\nResults Search: {json.dumps(results, indent=4, ensure_ascii=False)}\nThe Number Of Search Results: {len(results)}")
            return (False, )
    

    #? لإرجاع قيمة self.quran
    def quran_blocks(self):
        log(
                f'{__file__} > quran_blocks | Return All The Quran Data ', 
                f'The user return all data quran from json file'
                ).write_message()
        return self.quran
    

    #* تحميل صفحات القرآن الكريم
    def page_pic(
            self,
            page:list,
            type:str,
            path:str, 
            name_folder:str,
            return_imge=False):
        """
        page_pic(func): دالة تقوم على تحميل صفحات القرآن من الإنترنت
        page(list): تقوم بوضع عدد الأيات التي تريد تحميلها
        path(str): المسار تنزيل الصور
        name_folder(str): أسم الملف الذي ستكون فيها الصور الصفحات
        return_imge(bool): في حال تم تفعيل الخيار، سيتم تحميل الصفحات على شكل بايت (Binary)
        """
        #? التحقق من موجود إنترنت
        try:
            requests.get('https://github.com/oaokm')
            log(
            f"{__file__} > Quran > page_pic | Check The Internet Status",
            f"The internet is good"
            ).write_message()

            #* قراءة ملف الذي يحتوي على رابط صفحات القرأن كاملة وبجودة عالية
            quran_books = json.load(open('./DATA/quran_books.json', 'r', encoding='utf8'))
            try:
                log(
                    f"{__file__} > Quran > page_pic | Check For Read JSON File",
                    f"The file (quran_books.json) is good"
                ).write_message()
                
                web_pic = quran_books[type]
            except KeyError as e:
                log(
                    f"{__file__} > Quran > page_pic | Check For Read JSON File",
                    f"Error(keyError): {e}"
                ).write_message()
                print(f'[ Quran > page_pic | KeyError ]: {e}')
            #* في حالة إذا كان خاصية (return_imge) مفعلة يتم تسجيل محتويات الصورة كبِّت(صيغة ثنائية)
            pics = list()
            
            #? هنا تبدأ عملية الوصول للصفحات وتحميلها
            PATH = f'{path}/{name_folder}'
            print(f"# Download Pages From Quran | [{os.path.abspath(PATH)}]")
            for p in tqdm(page):
                #? التحقق من إذا كان المدخل لا يتخطى عدد صفحات القرأن
                if p <= 604:
                    #* عملية الإتصال بالموقع لسحب الصورة الصفحة
                    urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
                    url    = web_pic.format(p)
                    r_page = requests.get(url=url, verify=False)
                    
                    #* عملية إنشاء ملف تمهيدًا لتحميل الصفحات
                    
                    #? في حال كان خيار return_imge مفعل فسوف يتم تحميل الصور كبٍّت(النظام الثنائي)
                    if return_imge:
                        pics.append([p, r_page.content])
                    else:
                        #! في حال إذا الملف الذي أدخل المستخدم غير موجود على القرص سيتم تنزيل مباشرًا
                        if not os.path.exists(PATH):
                            os.makedirs(PATH)
                        #* نحميل الصور
                        with open(f"{PATH}/{p}.png", 'wb') as f:
                            f.write(r_page.content)
                            f.close()
                else:
                    print('I have exceeded 604 Page. Please adhere to the number of Page (from 1 to 604).')
            #? في جال إذا خيار return_imge مُفعل، يتم إرجاع قيمتها
            if return_imge:
                return pics
        #? في حال عدم وجود إنترنت يتم رفض الأمر
        except requests.exceptions.ConnectionError as e:
            log(
            f"{__file__} > Quran > page_pic | Check The Internet Status",
            f"The WiFi connection error, please check your internet"
            ).write_message()
            print(f"[ Quran | page_pic > The WiFi connection error ] {e}")

if __name__ == '__main__':
    print('[quran.py] This file is not to run')


'''
    MIT License | Copyright (C) 2023 Osamah Awadh (https://github.com/oaokm)
'''

import os
from .log import log
from .cheak import cheak

__version__   = '1.1.1'
__main_path__ = os.path.dirname(__file__)

os.chdir(path=__main_path__)


def Download_DATA():
    log(
        f'{__file__} | Download DATA Request ', 
        f'The user to been request the cheak and download files form Github'
        ).write_message()
    down       = cheak()
    list_files = down.find_DATA_folder()
    if list_files != []:
        down.download()
    else:
        print("[Download_DATA | Status Scan] All files are uploaded")

def where_me():
    log(
        f'{__file__} | Where Me Request ', 
        f'The user to been request the path program'
        ).write_message()
    return __main_path__

def show_me_log():
    log(
        f'{__file__} | Show Me Log Request ', 
        f'The user to been request the path log file'
        ).write_message()
    return f"{__main_path__}/DATA/loging.log"




from .quran import Quran
from .log import log
from tqdm import tqdm

class khatma:
    def __init__(self, days:int):
        log(
            f"{__file__} > khatma | Status The Class",
            f"True, days(int): {days}"
            ).write_message()
        self.days       = days
        self.werrd_page = list()
        self.quran      = Quran().quran_blocks()

    #* دالة تقوم على تقسيم صفحات القرآن لهدف إنهاء قراءة القرآن في يوم معين
    def Khatma_page(self, report=False, werrd=False):
        """
        Khatma_page(func): هي دالة تقوم بتقسيم صفحات القرآن الكريم بهدف إنهاء قراءتها
        report(bool): في حال تفعيل الخيار يتم طباعة تقرير عن النتائج التي تم تحليلها من هذه الدالة
        werrd(bool): في حال تفعيل الخيار تتم عملية تقسيم القرآن بناءًا على عدد الأيام الذي أدخلها المستخدم. في حال عدم تفعيل هذا الخيار يقوم البرنامج بإعادة قيمة عدد الصفحات اليومية الازمة لإنهاء قراءة القرآن كاملة
        """
        log(
            f"{__file__} > khatma > Khatma_page | Info",
            f"Days:{self.days}, report: {report}, werrd: {werrd}"
            ).write_message()
        #* عدد صفحات القرآن الكريم حسب نسخة مجمع الملك فهد لطباعة المصحف الشريف
        pages            = 604
        #* الأيام التي نريد ختم القرآن به
        Fdays            = self.days
        Fdays_pages      = list()
        #* بأستخدام الخوارزمية الجشعة لتقسيم صفحات القرأن الكريم
        deno             = [int(pages/Fdays), 2, 1]
        for day in deno:
            while day <= pages:
                Fdays_pages.append(day)
                pages -= day
            else:
                continue
        #* التحقق من أن عدد العناصر في Fdays_pages أكبر من عدد الأيام المدخلة
        if len(Fdays_pages) > self.days:
            #? هنا يقوم بعمل مُوازنة لعدد العناصر مع عدد الأيام في حال إذا كان عدد العناصر أكبر من عدد الأيام
            rest               = Fdays_pages[-(len(Fdays_pages) - Fdays):]
            Fdays_pages_N      = Fdays_pages[:-(len(Fdays_pages) - Fdays)]
            #? تحدث هنا عملية الموازنة لكي تصبح متساوية مع عدد الأيام المدخلة
            for i in range(len(rest)):
                Fdays_pages_N[i] += rest[i]
            report_pages_N = f"[REPORT KHATMA]\nDays: {self.days}\nReading Rate: {int(sum(Fdays_pages_N)/len(Fdays_pages_N))} (Page/Day)\nWeerd: {Fdays_pages_N}"
            #? التحقق من أن مجموع الصفحات المقسمة تساوي عدد صفحات القرآن الكريم
            if sum(Fdays_pages_N) == 604:
                self.werrd_page = Fdays_pages_N
                #* في حالة طلب المستخدم عدم تفعيل التقسيم على القرآن يتم تنفيذ هذه العملية
                if werrd == False:
                    if report: print(report_pages_N)
                    return Fdays_pages_N
            #! في حال وجود مشكلة يتم تنفيذ هذا الأمر، وهي مشكلة في عملية الموازنة
            else:
                log(
                f"{__file__} > khatma > Khatma_page | Partition Error ",
                f"An error has occurred that is not supposed to happen. Value: len(Fdays_pages) > self.days: {len(Fdays_pages) > self.days}, len(Fdays_pages):{len(Fdays_pages)}, Fdays_pages: {Fdays_pages}, days: {self.days}"
                ).write_message()
                print("[ khatma > Khatma_page | Partition Error ] An error has occurred that is not supposed to happen, Visit the library's Issues page: https://github.com/oaokm/AL-Khatma/issues")
        #* إذا كانت عملية التقسيم تساوية مع عدد الأيام يتم تنفيذ هذا الأمر مباشرًا
        else:
            report = f"[REPORT KHATMA]\nDays: {self.days}\nReading Rate: {int(sum(Fdays_pages)/len(Fdays_pages))} (Page/Day)\nWeerd: {Fdays_pages}"
            #? التحقق من أن مجموع الصفحات المقسمة تساوي عدد صفحات القرآن الكريم
            if sum(Fdays_pages) == 604:
                self.werrd_page = Fdays_pages
                if werrd == False:
                    if report: print(report)
                    return Fdays_pages
            else: 
                if werrd == False:
                    if report: print(report)
                    return Fdays_pages
        
        #* في حال تفعيل هذا الخيار، تبدأ عملية تقسيم القرآن الكريم على عدد الأيام المدخلة
        if werrd:
            """
            [تعريف المتغيرات]
            0. page_per_day(list): يكون عدد العناصر في هذا المتغير مساوي لعدد الأيام المدخلة، وكل عنصر يحتوي على عدد الأيات التي يجب قرأتها
            1. for_day(list): يتم تسجيل معلومات الأيات لكي تم لاحقًا إلى متغير page_per_day
            2. stop_id(int): هذا المتغير يساعد في عملية التقسيم، ووضيفته هو معرفة مُعرف الأية الكريمة لتتم عملية التقسيم بسهولة
            """
            page_per_day = list()
            for_day      = list()
            stop_id      = int() # int() = 0
            
            #* تبدا هنا عملية التقسيم
            print("The process of dividing the Quranic verses begins ... ")
            for i in tqdm(range(1, len(self.werrd_page)+1)):
                #? يأخذ quran قيمة self.quran مع آخر مُعرف تم التوقف عنه
                for quran in self.quran[stop_id:]:
                    for_day.append(quran)
                    #? في حال أن عدد الأيات أكبر من مجموع الأيات الموجودة في قائمة التقسيم يتم تنفيذ العملية
                    if quran['page'] > sum(self.werrd_page[:i]):
                        #* تسجيل آخر مُعرف 
                        stop_id = quran['ID']
                        #* تدوين الأيات إلى page_per_day + تصقير for_day + الإزاحة من عملية التكرار
                        page_per_day.append(for_day)
                        for_day = []
                        break
                    else:
                        continue
                #* تسجيل آخر قيمة (اليوم الأخير)
                if len(page_per_day) == len(self.werrd_page)-1:
                    page_per_day.append(self.quran[stop_id:])
                else:
                    continue
            report = f"[REPORT KHATMA]\nDays: {self.days}\nReading Rate: {int(sum(self.werrd_page)/len(self.werrd_page))} (Page/Day)\nWeerd: {self.werrd_page}\nNumber of Werrd: {len(page_per_day)}"
            if report: print(report)
            log(
                f"{__file__} > khatma > Khatma_page > (werrd) | Status",
                f"True"
                ).write_message()
            #* إرجاع قيمة العملية
            return page_per_day
        else: pass



import requests
import os
import json
from .log import log
from tqdm import tqdm

main_path = os.path.dirname(__file__)

class cheak:

    def __init__(self):
        self.download_file = list()
        self.path_url      = list()
        # self.JSONFILE      = json.load(open('./cheak_download.json', 'r', encoding='utf8'))
        try:
            self.JSONFILE      = requests.get(
                url="https://raw.githubusercontent.com/oaokm/AL-Khatma/main/cheak_download.json"
                ).json()
        except requests.exceptions.ConnectionError as e:
            log(
            f"{__file__} > cheak > __init__ | Check The Internet Status",
            f"The WiFi connection error, please check your internet"
            ).write_message()
            print(f"[ cheak | __init__ > The WiFi connection error ] {e}")

    def find_DATA_folder(self, showme_log=False):
        for block in self.JSONFILE:
            path = f"{main_path}/{block['name_folder']}"
            for ch in block['files']:
                cheak_file = os.path.exists(path=f"{path}/{ch}")
                
                if showme_log: print(f'[{path}/{ch}] {cheak_file}')
                
                if not cheak_file: self.download_file.append(f'{path}/{ch}')
            else:
                continue
        
        
        for i in range(len(self.download_file)):
            text = self.download_file[i].split('/')
            for y in range(len(text)):
                if text[-y-1] == "AL_Khatma":
                    self.path_url.append("/".join(text[-y:]))
                    break
                else: continue
        return self.download_file
    
    def download(self):
        if self.path_url != []:
            print(f"# Download DATA Folder in {os.path.dirname(self.download_file[0])}")
            
            for download in tqdm(range(len(self.path_url))):
                if not os.path.exists(os.path.dirname(self.download_file[download])):
                    os.makedirs(os.path.dirname(self.download_file[download]))
                github_file = requests.get(
                        url=f"https://raw.githubusercontent.com/oaokm/AL-Khatma/main/{self.path_url[download]}"
                            )
                if github_file.status_code == 200:
                    with open(self.download_file[download], 'wb') as f:
                        f.write(github_file.content)
                        f.close()
                        
                else:
                    print(f"[INFO]\nURL: {github_file.url}\tStatus Code: {github_file.status_code}")
        else:
            print("[cheak.py] Status folder is very good")

if __name__ == '__main__':
    c = cheak()
    c.find_DATA_folder()
    c.download()


from datetime import datetime as dt
import os

__main_path__ = os.path.dirname(__file__)

def nowing():
    return dt.now().timestamp()

class log:

    def __init__(self, title:str, mess:str):
        if os.path.exists(path=f'{__main_path__}/DATA/loging.log'):
            self.log      = open(f'{__main_path__}/DATA/loging.log', 'a+', encoding='utf8')
            self.title    = title
            self.mess     = mess
            self.log_list = self.log.read().split('\n')
        else:
            try:
                os.makedirs(f'{__main_path__}/DATA/')
            except FileExistsError:
                pass
            self.log      = open(f'{__main_path__}/DATA/loging.log', 'a+', encoding='utf8')
            self.title    = title
            self.mess     = mess
            self.log_list = self.log.read().split('\n')

    def write_message(self, time=True):
        
        if time: self.log.write(f"\n[{self.title}|({nowing()})] {self.mess}")
        
        else: self.log.write(f"\n[{self.title}] {self.mess}")

